%Garcia-Jimeno, Camilo. "The Political Economy of Moral Conflict: An
%Empirical Study of Learning and Law Enforcement under Prohibition"
%Econometrica

%Main file

clear all
close all
clc

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%Import data
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%Open data file structural_dataset.mat
load structural_dataset.mat

global years cities BIG latent_classes

BIG = BIGG;
cities = citiess;
years = yearss;

%% Model Choices

neighbors=0;                                                               % =0 for model with only local learning
                                                                           % =1 for model with learning from neighbors

latent_classes = 2;                                                        %Number of Latent classes

                                                                                  
%%    %MLE                                                                            
                                                                              
%Structural parameters:

%z: 13: f(share_pop_15-44, a_Foreign_white, a_Black, baptist_pi, orthodox_pi, evangelical_pi, jew_pi, mormon_pi, lutheran_pi, methodist_pi, catholic_pi, presbyterian_pi, constant)
%alpha: 13: f(Constant, Dry_Law, Enforcement_Law, Prohibition_seat_dummy, PR_dummy, WA_dummy, JK_dummy, DT_dummy, CH_dummy, KS_dummy, SF_dummy, LA_dummy SE_dummy)
%Bias: 10: f(Baptist, Orthodox, Evangelical, Jew, Mormon, Lutheran, Methodist, Catholic, Presbyterian, Constant)
%kappa: 1
%lambda: 1
%xchi: 2
%var_q: 2
%var_zet: 2
%rho: 2
%a: 2
%b: 2: f(gamma*z)
%theta_true: 14: f(Border_dummy South _dummy, State_capita_dummy, pop_15-44, foreign_white, Black, constant)
%eta: 2
%var_xi: 1

%Definition of Endogenous variables
 poli = p_detrend_rescale;
 drunk = d;

	%Main Model
    if neighbors==0
        mean_crime = mean(logQ);
        crime = logQ - mean_crime;
    else
        Dist_close = zeros(cities,cities);                   %Matrix of distance weights
        for i=1:cities
            for j=1:cities
                if DIST(i,j)<200
                    Dist_close(i,j) = 1;
                else
                    Dist_close(i,j) = 0;
                end
            end
        end
    
        Dist_close = Dist_close./(repmat(sum(Dist_close,2),1,66));
        crime00 = (Dist_close*reshape(logQ,19,66)')';
        crime0 = crime00(:);
        mean_crime = mean(crime0);
        crime = crime0 - mean_crime;

    end


%% %EM Algorithm
num_param = 37 + latent_classes*16;

% Alcohol Supply

x0 = [-1 -1];

[supply_par,~,exitflag,output] = supply(drunk, YUP, x0);

MLE_estimates = zeros(num_param,1000);
fval = zeros(1,1000);
z=zeros(cities,latent_classes,1000);

ii=2;
z(:,:,1) = (1/latent_classes)*ones(cities,latent_classes);            %Initial latent class probabilities (uniform distribution)

%Starting Point for Search Minimization
    %       [   z_bar(1:13)     alpha(14:26)     bias(27:36)    xchi1(37:38)         k(39:40)                 lam(41:42)             var_q(43:44)   var_zet(45:46)   rho(47:48)  gamma_a(49:50)  gamma_b(51:52)   theta_true(53:66)     eta(67:68)    var_xi(69)]
       
        THETA = [ 0.1*ones(1,13)   0.5*ones(1,13)  -0.5*ones(1,10)  100 100   	supply_par(1)*ones(1,2)  supply_par(2)*ones(1,2)	    1 1  	       1 1  	       0 0  	    2 5  	       5 5  	        1*ones(1,14)	   -10 -10  	    0.1  ]';    
        
MLE_estimates(:,1) = THETA;
fval(1)=-10;

for ii=2:1000

%Run the optimization procedure
[MLE_estimates(:,ii),fval(ii),exitflag,output] = nested(z(:,:,ii-1), poli, drunk, crime, mean_crime, YUP, P, R, X_char, Abar15_44, E, E_bar, R_o, L, supply_par, THETA);

THETA = MLE_estimates(:,ii);

new_z = update_z(z(:,:,ii-1), poli, drunk, crime, mean_crime, YUP, P, R, X_char, Abar15_44, E, E_bar, R_o, L, THETA); 
     
z(:,:,ii) = new_z;

if abs(fval(ii) - fval(ii - 1))< 0.0001
    break
end

end
           
MLE_final = MLE_estimates(:,ii);


%% %Recover Belief Sequences, Densities of Errors, and Likelihoods:

[beliefs varrho_med g_eps f] = estimates(poli, drunk, crime, mean_crime, P, YUP, Abar15_44, E, E_bar, L, R, X_char, R_o, MLE_final);

    beliefs = sum(kron(z(:,:,ii),ones(years,1)).*beliefs,2);

        belief_seq = reshape(beliefs,years,cities);

    g_eps = sum(kron(z(:,:,ii),ones(years,1)).*g_eps,2);

    varrho_med = min(sum(kron(z(:,:,ii),ones(years,1)).*varrho_med,2),0);

    log_like = sum(log(sum((f.*kron(z(:,:,ii),ones(years,1))),2)));
     
%%  Compute Analytic Standard Errors

K = num_param;

S=zeros(K,years*cities);

for j=1:K

	S(j,:) = (1/0.00001)*(score(z(:,:,ii), poli, drunk, crime, mean_crime, YUP, P, R, X_char, Abar15_44, E, E_bar, R_o, L, MLE_final + EPSILON(j,:)') - score(z(:,:,ii), poli, drunk, crime, mean_crime, YUP, P, R, X_char, Abar15_44, E, E_bar, R_o, L, MLE_final))';

end

Outer=zeros(K,K,years*cities);

for i=1:years*cities

	Outer(:,:,i) = S(:,i)*S(:,i)';

end

Avar_inv = sum(Outer,3);

Avar_inv =(Avar_inv(1:K-1,1:K-1));

Avar = (inv(Avar_inv));

stderr = sqrt(diag(Avar)); 


%% %Parameters (Table 2)

    [z_bar_mean z_bar_stdev alpha_mean alpha_stdev bias_mean bias_stdev theta_true_mean theta_true_stdev...
    a_mean a_stdev b_mean b_stdev xchi_mean xchi_stdr k k_stdr lam lam_stdr var_q var_q_stdr var_zet var_zet_stdr...
    var_xi var_xi_stdr rho rho_stdr eta eta_stdr PI1 PI2] = param_table(Abar15_44, E, E_bar, L, R, X_char, R_o, MLE_final, z(:,:,ii), Avar);

disp('PARAMETER ESTIMATES')

disp('Common Across Latent Classes')
    disp('Mean Average Moral Views (z)')
    disp([z_bar_mean; z_bar_stdev])

    disp('Mean Enforcement Elasticity (alpha)')
    disp([alpha_mean; alpha_stdev])

    disp('Mean Prior Bias (B)')
    disp([bias_mean; bias_stdev])

disp('Latent Class Specific')
    disp('Mean Crime Elasticity (theta)')
    disp([theta_true_mean; theta_true_stdev])

    disp('Wetness Distribution Parameter (a)')
    disp([a_mean; a_stdev])

    disp('Wetness Distribution Parameter (b)')
    disp([b_mean; b_stdev])

disp('Other Parameters')
    disp('Mean Arrest Probability Parameter (xchi)')
    disp([xchi_mean xchi_stdr])

    disp('Mean Marginal utility of Public Goods (eta)')
    disp([eta eta_stdr])

    disp('Alcohol Supply Recovery Parameter (kappa)')
    disp([k k_stdr])

    disp('Alcohol Supply Recovery Parameter (lambda)')
    disp([lam lam_stdr])

    disp('Variance of the Crime Equation Residual (var_q)')
    disp([var_q var_q_stdr])

    disp('Variance of Idiosyncratic Moral Views (var_zet)')
    disp([var_zet var_zet_stdr])

    disp('Variance of Idiosyncratic Prior Biases (var_xi)')
    disp([var_xi var_xi_stdr])

    disp('Correlation between Idiosyncratic Moral Views and Prior Biases (rho)')
    disp([rho rho_stdr])

    disp('Fraction of Cities in Each Latent Class')
    disp([PI1 PI2])


%% %Coefficients (Table 3)

[z_bar_coeff z_bar_stdr alpha_coeff alpha_stdr bias_coeff bias_stdr b_coeff b_stdr theta_coeff theta_stdr] = coeff_table(MLE_final, Avar);

disp('COEFFICIENT ESTIMATES')

disp('Coefficient Estimates and Standard Errors for Moral Views (z) Covariates')
disp([z_bar_coeff'; z_bar_stdr'])

disp('Coefficient Estimates and Standard Errors for Enforcement Elasticity (alpha) Covariates')
disp([alpha_coeff'; alpha_stdr'])

disp('Coefficient Estimates and Standard Errors for Prior Bias (B) covariates')
disp([bias_coeff'; bias_stdr'])

disp('Average (across latent classes) Coefficient Estimates and Standard Errors for Crime Elasticity (theta) Covariates')
disp('Latent Class 1')
disp([theta_coeff(:,1)'; theta_stdr(:,1)'])

disp('Latent Class 2')
disp([theta_coeff(:,2)'; theta_stdr(:,2)'])

disp('Coefficient Estimates and Standard Errors for wetness distribution elasticity to Moral Views (gamma)')
disp('Latent Class 1  Latent Class 2')
disp([b_coeff; b_stdr])


%%  %Model Fit (Table 4 and Figure 9)

%Model Fit Table
[CRIME_fit DRUNK_fit POLI_fit d_hat q_hat p_hat] = fit(poli, drunk, crime, mean_crime, P, YUP, Abar15_44, E, E_bar, L, R, X_char, R_o, MLE_final, z(:,:,ii));

%Data for Law Enforcement time series fit
p_time=reshape(poli,years,cities);
p_time_hat=p_hat;

d_time=reshape(drunk,years,cities);
d_time_hat=d_hat;

q_time=reshape(logQ,years,cities);
q_time_hat=q_hat;

%Data for Fit Table

disp('MODEL FIT')

disp('Law Enforcement Data and Model Pre and Data and Model Prohib')
disp(POLI_fit)

disp('Drunkenness Arrest Rate Data and Model Pre and Data and Model Prohib')
disp(DRUNK_fit)

disp('Log Homicide Rate Data and Model Pre and Data and Model Prohib')
disp(CRIME_fit)


%%   %Out of sample (Figure 10)

    mean_crime_pre = sum((crime + mean_crime).*(1-P))/sum(1-P);

    mean_crime_pro = sum((crime + mean_crime).*P)/sum(P);

[Out_model_p Out_data_p Out_model_q Out_data_q Out_model_bel] = outofsample(poli, drunk, crime, mean_crime_pre, mean_crime_pro, mean_crime, P, YUP, Abar15_44, E, E_bar, L, R, X_char, R_o, MLE_final, z(:,:,ii), p_rescale30_36, logQ_30_36);


%% %Counterfactual Exercises (Table 5) (Results may differ slightly from those reported in the table due to sampling variation)

%0. Benchmark
    counterf=0;

    [Bench_p Bench_d Bench_q Bench_bel Bench_cap_omegas Bench_p_pre_mean Bench_p_pro_mean Bench_d_pre_mean Bench_d_pro_mean Bench_q_pre_mean Bench_q_pro_mean Bench_bel_pre_mean Bench_bel_pro_mean] = counterfactual_seq(poli, drunk, crime, mean_crime_pre, mean_crime_pro, mean_crime, P, YUP, Abar15_44, E, E_bar, L, R, X_char, R_o, MLE_final, z(:,:,ii), counterf);

    %Yearly belief averages
    disp('Table 5 Benchmark Counterfactual')
    Bench_Mean_beliefs = mean(Bench_bel,2);
    disp('1929 Mean belief')
    disp(Bench_Mean_beliefs(end))

    [Bench_SW Bench_stdev_U Bench_Max_U Bench_Min_U] = social_welfare(Bench_p, Bench_d, Bench_q, Bench_bel, Bench_cap_omegas, P, YUP, Abar15_44, E, E_bar, L, R, X_char, R_o, ln_pop_bar, MLE_final, z(:,:,ii), counterf);

    disp('Total Ex Ante Utilitarian Social Welfare')
    disp(Bench_SW)
    disp('Standard Deviation of Utilites across all Citizens')
    disp(Bench_stdev_U)
    disp('Best Off Citizen')
    disp(Bench_Max_U)
    disp('Worst Off Citizen')
    disp(Bench_Min_U)
    

%1. Never Prohibition
    counterf=1;

    P_never = zeros(years*cities,1);
    YUP_never = zeros(years*cities,1);

    [Never_p Never_d Never_q Never_bel Never_cap_omegas Never_p_pre_mean Never_p_pro_mean Never_d_pre_mean Never_d_pro_mean Never_q_pre_mean Never_q_pro_mean Never_bel_pre_mean Never_bel_pro_mean] = counterfactual_seq(poli, drunk, crime, mean_crime_pre, mean_crime_pro, mean_crime, P_never, YUP_never, Abar15_44, E, E_bar, L, R, X_char, R_o, MLE_final, z(:,:,ii), counterf);

    C_1_p_pre = (Never_p_pre_mean-((Bench_p_pre_mean+Bench_p_pro_mean)/2))/((Bench_p_pre_mean+Bench_p_pro_mean)/2);
    
    C_1_d_pre =(Never_d_pre_mean-((Bench_d_pre_mean+Bench_d_pro_mean)/2))/((Bench_d_pre_mean+Bench_d_pro_mean)/2);
    
    C_1_q_pre = (Never_q_pre_mean-((Bench_q_pre_mean+Bench_q_pro_mean)/2))/((Bench_q_pre_mean+Bench_q_pro_mean)/2);

    disp('Table 5 Counterfactual 1')
    disp([C_1_p_pre C_1_d_pre C_1_q_pre])
 
    
    [Never_SW Never_stdev_U Never_Max_U Never_Min_U] = social_welfare(Never_p, Never_d, Never_q, Never_bel, Never_cap_omegas, P_never, YUP_never, Abar15_44, E, E_bar, L, R, X_char, R_o, ln_pop_bar, MLE_final, z(:,:,ii), counterf);

    disp('Total Ex Ante Utilitarian Social Welfare')
    disp(Never_SW)
    disp('Standard Deviation of Utilites across all Citizens')
    disp(Never_stdev_U)
    disp('Best Off Citizen')
    disp(Never_Max_U)
    disp('Worst Off Citizen')
    disp(Never_Min_U)
    
          
%2. No learning (stubborn)
    counterf=2;

    [NoLearn_p NoLearn_d NoLearn_q NoLearn_bel NoLearn_cap_omegas NoLearn_p_pre_mean NoLearn_p_pro_mean NoLearn_d_pre_mean NoLearn_d_pro_mean NoLearn_q_pre_mean NoLearn_q_pro_mean NoLearn_bel_pre_mean NoLearn_bel_pro_mean] = counterfactual_seq(poli, drunk, crime, mean_crime_pre, mean_crime_pro, mean_crime, P, YUP, Abar15_44, E, E_bar, L, R, X_char, R_o, MLE_final, z(:,:,ii), counterf);
    
    C_3_p_pro = (NoLearn_p_pro_mean-Bench_p_pro_mean)/Bench_p_pro_mean;
    
    C_3_d_pro =(NoLearn_d_pro_mean-Bench_d_pro_mean)/Bench_d_pro_mean;
    
    C_3_q_pro = (NoLearn_q_pro_mean-Bench_q_pro_mean)/Bench_q_pro_mean;

    disp('Table 5 Counterfactual 2')
    disp([C_3_p_pro C_3_d_pro C_3_q_pro])


    [NoLearn_SW NoLearn_stdev_U NoLearn_Max_U NoLearn_Min_U] = social_welfare(NoLearn_p, NoLearn_d, NoLearn_q, NoLearn_bel, NoLearn_cap_omegas, P, YUP, Abar15_44, E, E_bar, L, R, X_char, R_o, ln_pop_bar, MLE_final, z(:,:,ii), counterf);

    disp('Total Ex Ante Utilitarian Social Welfare')
    disp(NoLearn_SW)
    disp('Standard Deviation of Utilites across all Citizens')
    disp(NoLearn_stdev_U)
    disp('Best Off Citizen')
    disp(NoLearn_Max_U)
    disp('Worst Off Citizen')
    disp(NoLearn_Min_U)
    
    
     
%3. Polarization
    counterf=3;

    [Polar_p Polar_d Polar_q Polar_bel Polar_cap_omegas Polar_p_pre_mean Polar_p_pro_mean Polar_d_pre_mean Polar_d_pro_mean Polar_q_pre_mean Polar_q_pro_mean Polar_bel_pre_mean Polar_bel_pro_mean] = counterfactual_seq(poli, drunk, crime, mean_crime_pre, mean_crime_pro, mean_crime, P, YUP, Abar15_44, E, E_bar, L, R, X_char, R_o, MLE_final, z(:,:,ii), counterf);
 
    Polar_Mean_beliefs = mean(Polar_bel,2);
    disp('Mean belief Sequence')
    disp(Polar_Mean_beliefs)

    
    C_4_p_pro = (Polar_p_pro_mean-Bench_p_pro_mean)/Bench_p_pro_mean;
    
    C_4_d_pro =(Polar_d_pro_mean-Bench_d_pro_mean)/Bench_d_pro_mean;
    
    C_4_q_pro = (Polar_q_pro_mean-Bench_q_pro_mean)/Bench_q_pro_mean;

    disp('Table 5 Counterfactual 3')
    disp([C_4_p_pro C_4_d_pro C_4_q_pro])
   

    [Polar_SW Polar_stdev_U Polar_Max_U Polar_Min_U] = social_welfare(Polar_p, Polar_d, Polar_q, Polar_bel, Polar_cap_omegas, P, YUP, Abar15_44, E, E_bar, L, R, X_char, R_o, ln_pop_bar, MLE_final, z(:,:,ii), counterf);

    disp('Total Ex Ante Utilitarian Social Welfare')
    disp(Polar_SW)
    disp('Standard Deviation of Utilites across all Citizens')
    disp(Polar_stdev_U)
    disp('Best Off Citizen')
    disp(Polar_Max_U)
    disp('Worst Off Citizen')
    disp(Polar_Min_U)
    
    
    









