%Function for Counterfactual Simulations

function [p_model d_model q_model bel_model cap_omegas_model p_model_pre_mean p_model_pro_mean d_model_pre_mean d_model_pro_mean q_model_pre_mean q_model_pro_mean bel_model_pre_mean bel_model_pro_mean] = counterfactual_seq(poli, drunk, crime, mean_crime_pre, mean_crime_pro, mean_crime, P, YUP, Abar15_44, E, E_bar, L, R, X_char, R_o, THETA, z, counterf)

    global years cities latent_classes 

    
        %Recover parameters and densities:
            %Structural parameters as functions of the data
        
            %Common across Latent Classes
                z_bar = log(exp([Abar15_44 E(:,2:3) R L(:,1)]*THETA(1:13)) + 1);                                                                                                      %Mean moral beliefs are a function of religious distribution, time varying as religious distribution changes: R = [baptist orth evangel jew mormon luth metho catho presby] (baptist is ommited category) NxT,1   
            
                alpha = 1 + log(exp(L*THETA(14:26)) + 1);                                                                                                                             %Enforcement parameter is a function of: Pre-prohibition trend and L=Dry laws, enfocement law dummy, prohibition field headquarter, prohibition field dummies) NxT,1
               
                bias = [R_o L(:,1)]*THETA(27:36);                                                                                                                                     %Structural prior bias
        
             %Normalization                                                                                                                                         
                var_xi = 0.1;
               
        
            %Latent-Class specific                
                xchi = log(exp(( repmat(THETA(36 + 1:36 + latent_classes)',years*cities,1) ) ) + 1);                                                                                  %Drunkard Arrest enforcement parameter                              
        
                k = exp(THETA(36 + latent_classes + 1:36 + 2*latent_classes)')./(1 + exp(THETA(36 + latent_classes + 1:36 + 2*latent_classes)') );                                    %Speed of alcohol-market catchup parameter-inverse of trough 
            
                lam = exp(THETA(36 + 2*latent_classes + 1:36 + 3*latent_classes)')./(1 + exp(THETA(36 + 2*latent_classes + 1:36 + 3*latent_classes)') );                              %Impact fall in alcohol market parameter 

                var_q = log(exp(THETA(36 + 3*latent_classes + 1:36 + 4*latent_classes)') + 1);                                                                                        %Variance of the error in the crime equation
        
                var_zet = log(exp(THETA(36 + 4*latent_classes + 1:36 + 5*latent_classes)') + 1);                                                                                      %Variance of the distribution of moral shocks
       
                rho = ((exp(THETA(36 + 5*latent_classes + 1:36 + 6*latent_classes)')-1)./(1+exp(THETA(36 + 5*latent_classes + 1:36 + 6*latent_classes)')));                           %Correlation between morals and prior beliefs
        
                a = log(exp(repmat(THETA(36 + 6*latent_classes + 1:36 + 7*latent_classes)',years*cities,1)) + 1) + 1;                                                                 %Wetness distribution parameter (correlated with moral views)                                                                                                                                                                            
                
                    if counterf==3                                          %Increase wet share for Polarization Counterfactual to 3/4 on average
                        a = 3*a;
                    end
                                
                b = log(exp(repmat(THETA(36 + 7*latent_classes + 1:36 + 8*latent_classes)',years*cities,1).*repmat(z_bar,1,latent_classes)) + 1) + 1;                                 %Wetness distribution parameter (correlated with moral views)
                    
                theta_true = (log(exp([X_char(:,31:33) Abar15_44 E_bar(:,2:3) L(:,1)]*reshape(THETA(36 + 8*latent_classes + 1:36 + 15*latent_classes),7,latent_classes)) + 1));       %Structural elasticity of crime to Prohibition                                                                                                 
        
                eta = exp(THETA(36 + 15*latent_classes + 1:36 + 16*latent_classes)')./(1 + exp(THETA(36 + 15*latent_classes + 1:36 + 16*latent_classes)'));                           %Marginal utility of public goods

                    
    beliefs = zeros(years*cities,latent_classes);
    cap_omega = zeros(years*cities,latent_classes);
    varrho_med = zeros(years*cities,latent_classes);
   
     
    for i = 1:latent_classes

        %Compute the vector of common beliefs sequences and belief weights sequences
        [beliefs(:,i), cap_omega(:,i)] = public_beliefs(poli,drunk,crime,mean_crime,P,alpha,bias,var_xi,var_q(i),xchi(i));
       
        varrho_med(:,i) = g_varrho(poli,YUP,P,beliefs(:,i),alpha,a(:,i),b(:,i),k(i),lam(i),z_bar,eta(i));
        
    end
    
                  
    
 %Organize parameters
    
 num_draws = 1000;   
 
 P=reshape(P,years,cities);
 
 YUP=reshape(YUP,years,cities);
 


 alpha = reshape(alpha,years,cities);
 
    if counterf==1
        alpha = repmat(mean(alpha),years,1);                                %For No Prohibition or No Learning Counterfactuals fix alpha over time
    end
    
 z_bar = reshape(z_bar,years,cities);
 
    if counterf==3                                                          %Increase moral views for Polarization Counterfactual
        z_bar = repmat(prctile(z_bar,0.75),years,1);
    end
 
 bias = reshape(bias,years,cities);
    bias = bias(1,:);
 
 xchi = xchi(1,:);
 
 k = k(1);
 
 lam = lam(1);

 
 
    
%% %Latent Class 1
 a_LC1 = reshape(a(:,1),years,cities);
 
 b_LC1 = reshape(b(:,1),years,cities);
 
 theta_true_LC1 = reshape(theta_true(:,1),years,cities);
        
        
 BEL=zeros(years+1,cities,num_draws);                                          %Matrix to Fill in with belief sequences       
 OMEGAS=zeros(years,cities,num_draws);                                         %Matrix to fill in with Omegas sequences
 SUM_OMEGAS_SQ=zeros(years+1,cities,num_draws);                                %Matrix to fill in with sum of Squared Omegas
 SUM_Q_OMEGAS=zeros(years+1,cities,num_draws);                                 %Matrix to fill in with sum of (q - q_noProhib)*omegas
 
 CAP_OMEGAS=zeros(years+1,cities,num_draws);                                   %Matrix to fill in with Cap Omegas sequences

 
 for i=1:num_draws
     BEL(1,:,i)= bias;                                                         %Fill in 1911 (prior) with  bias    
 
     CAP_OMEGAS(1,:,i)=var_xi;                                                 %Fill in 1911 Cap omega with baseline
 end
 
 
 POL_LC1=zeros(years,cities,num_draws);                                        %Matrix to fill in with police sequences
 DRUNK_LC1=zeros(years,cities,num_draws);                                      %Matrix to fill in with drunkenness arrest sequences
 HOMI_LC1=zeros(years,cities,num_draws);                                       %Matrix to fill in with homicide sequences
 
       
 
        zzz_LC1 = kron(z(:,1),ones(years,1)).*varrho_med(:,1);
     
        pool_LC1_pre =  zzz_LC1.*(1-reshape(P,years*cities,1));
     
        pool_LC1_pre(pool_LC1_pre==0) = [];
        
        pool_LC1_pro = zzz_LC1.*reshape(P,years*cities,1);
        
        pool_LC1_pro(pool_LC1_pro==0) = [];
              
        VARRHO_LC1_pre = reshape(randsample(pool_LC1_pre,years*cities*num_draws,true),years,cities,num_draws);  %Matrices to fill in with mean unobserved median voter types
           
                                   
        if counterf==1
            VARRHO_LC1_pro = zeros(years,cities,num_draws);
        else 
            VARRHO_LC1_pro = reshape(randsample(pool_LC1_pro,years*cities*num_draws,true),years,cities,num_draws);
        end
                
        
        MU_LC1_draws = zeros(years,cities,num_draws);
        for i=1:num_draws
            MU_LC1_draws(:,:,i) = betarnd(a_LC1,b_LC1);                                 %Matrices to fill in with wet share draws 
        end
 
       
            eps_LC1_draws = sqrt(var_q(1)).*randn(years*cities*num_draws,1);  %Matrices to fill in with crime error draws 
          
            eps_LC1_draws = reshape(eps_LC1_draws,years,cities,num_draws);
        
        
   %This loop computes the sequences of simulated outcomes and beliefs
    
   for i=1:num_draws
        for c=1:cities
            for t=1:years
                        
            POL_LC1(t,c,i) = (1/(alpha(t,c) - 1))*(log(alpha(t,c)*K_t(YUP(t,c),k,lam)/eta(1)) + log((a_LC1(t,c)/(a_LC1(t,c)+b_LC1(t,c)))*(z_bar(t,c) - P(t,c)*BEL(t,c,i) + 1) + (1-P(t,c))*VARRHO_LC1_pre(t,c,i) + P(t,c)*VARRHO_LC1_pro(t,c,i) ) - 1);
         
                if isreal(POL_LC1(t,c,i))==0
                    POL_LC1(t,c,i) = 0;
                end
            
            %HOMI_LC1(t,c,i) = mean_crime_pre*(1-P(t,c)) + mean_crime_pro*P(t,c) + K_t(YUP(t,c),k,lam)*MU_LC1_draws(t,c,i)*( exp(-alpha(t,c)*POL_LC1(t,c,i)) + P(t,c)*theta_true_LC1(t,c)*(1 - exp(-alpha(t,c)*POL_LC1(t,c,i)))) + eps_LC1_draws(t,c,i); 
             HOMI_LC1(t,c,i) = K_t(YUP(t,c),k,lam)*MU_LC1_draws(t,c,i)*( exp(-alpha(t,c)*POL_LC1(t,c,i)) + P(t,c)*(bias(c)+theta_true_LC1(t,c))*(1 - exp(-alpha(t,c)*POL_LC1(t,c,i)))) + eps_LC1_draws(t,c,i);
             
                if abs(HOMI_LC1(t,c,i))>=5 && t>=2
                    HOMI_LC1(t,c,i) = HOMI_LC1(t-1,c,i);
                end
                
            DRUNK_LC1(t,c,i) = MU_LC1_draws(t,c,i)*K_t(YUP(t,c),k,lam)./(xchi(1) + exp(alpha(t,c).*POL_LC1(t,c,i)));
            
            OMEGAS(t,c,i) = P(t,c)*(K_t(YUP(t,c),k,lam)*MU_LC1_draws(t,c,i)*(1 - exp(-alpha(t,c)*POL_LC1(t,c,i))));
            
                if counterf==2
                    OMEGAS(t,c,i)=0;
                end
            
            SUM_OMEGAS_SQ(t+1,c,i) = SUM_OMEGAS_SQ(t,c,i) + (OMEGAS(t,c,i)*OMEGAS(t,c,i));
         
            CAP_OMEGAS(t+1,c,i) = 1/((1/var_xi) + ((1/var_q(1))*(SUM_OMEGAS_SQ(t+1,c,i))));
       
            SUM_Q_OMEGAS(t+1,c,i) = SUM_Q_OMEGAS(t,c) + (mean_crime + HOMI_LC1(t,c,i) - K_t(YUP(t,c),k,lam)*MU_LC1_draws(t,c,i)*exp(-alpha(t,c)*POL_LC1(t,c,i)))*OMEGAS(t,c,i);
        
            BEL(t+1,c,i) = CAP_OMEGAS(t+1,c,i)*(((1/var_xi)*BEL(1,c,i)) + ((1/var_q(1))*SUM_Q_OMEGAS(t+1,c,i)));
         
            end
       end
   end
 
  
   
  POL_LC1 = sum(POL_LC1,3)/num_draws; 
  DRUNK_LC1 = sum(DRUNK_LC1,3)/num_draws;
  HOMI_LC1 = sum(HOMI_LC1,3)/num_draws; 
   
  BEL_LC1 = sum(BEL,3)/num_draws;                                                            
    BEL_LC1 = BEL_LC1(1:years,:);                                           %Stop in 1933
 
 for t=2:years
     for c=1:cities
         if abs(BEL_LC1(t,c))>=5
             BEL_LC1(t,c)=BEL_LC1(t-1,c);
         end
     end
 end
      
  CAP_OMEGAS_LC1 = sum(CAP_OMEGAS,3)/num_draws;                                              
    CAP_OMEGAS_LC1 = CAP_OMEGAS_LC1(1:years,:);                             %Stop in 1933
       
    
    
%% %Latent Class 2    
 a_LC2 = reshape(a(:,2),years,cities);
    
 b_LC2 = reshape(b(:,2),years,cities);
 
 theta_true_LC2 = reshape(theta_true(:,2),years,cities);  
        
        
 BEL=zeros(years+1,cities,num_draws);                                          %Matrix to Fill in with belief sequences       
 OMEGAS=zeros(years,cities,num_draws);                                         %Matrix to fill in with Omegas sequences
 SUM_OMEGAS_SQ=zeros(years+1,cities,num_draws);                                %Matrix to fill in with sum of Squared Omegas
 SUM_Q_OMEGAS=zeros(years+1,cities,num_draws);                                 %Matrix to fill in with sum of (q - q_noProhib)*omegas
 
 CAP_OMEGAS=zeros(years+1,cities,num_draws);                                   %Matrix to fill in with Cap Omegas sequences

 
 for i=1:num_draws
     BEL(1,:,i)= bias;                                                         %Fill in 1911 (prior) with  bias    
 
     CAP_OMEGAS(1,:,i)=var_xi;                                                 %Fill in 1911 Cap omega with baseline
 end
 
 
 POL_LC2=zeros(years,cities,num_draws);                                        %Matrix to fill in with police sequences
 DRUNK_LC2=zeros(years,cities,num_draws);                                      %Matrix to fill in with drunkenness arrest sequences
 HOMI_LC2=zeros(years,cities,num_draws);                                       %Matrix to fill in with homicide sequences
 
       
 
        zzz_LC2 = kron(z(:,2),ones(years,1)).*varrho_med(:,2);
     
        pool_LC2_pre =  zzz_LC2.*(1-reshape(P,years*cities,1));
     
        pool_LC2_pre(pool_LC2_pre==0) = [];
        
        pool_LC2_pro = zzz_LC2.*reshape(P,years*cities,1);
        
        pool_LC2_pro(pool_LC2_pro==0) = [];
              
        VARRHO_LC2_pre = reshape(randsample(pool_LC2_pre,years*cities*num_draws,true),years,cities,num_draws);  %Matrices to fill in with mean unobserved median voter types
 
       
        if counterf==1
            VARRHO_LC2_pro = zeros(years,cities,num_draws);
        else 
             VARRHO_LC2_pro = reshape(randsample(pool_LC2_pro,years*cities*num_draws,true),years,cities,num_draws);
        end
        
        
        MU_LC2_draws = zeros(years,cities,num_draws);
        for i=1:num_draws
            MU_LC2_draws(:,:,i) = betarnd(a_LC2,b_LC2);                                 %Matrices to fill in with wet share draws 
        end
 
       
            eps_LC2_draws = sqrt(var_q(2)).*randn(years*cities*num_draws,1);  %Matrices to fill in with crime error draws 
          
            eps_LC2_draws = reshape(eps_LC2_draws,years,cities,num_draws);
        
        
   %This loop computes the sequences of simulated outcomes and beliefs
    
   for i=1:num_draws
        for c=1:cities
            for t=1:years
                        
            POL_LC2(t,c,i) = (1/(alpha(t,c) - 1))*(log(alpha(t,c)*K_t(YUP(t,c),k,lam)/eta(2)) + log((a_LC2(t,c)/(a_LC2(t,c)+b_LC2(t,c)))*(z_bar(t,c) - P(t,c)*BEL(t,c,i) + 1) + (1-P(t,c))*VARRHO_LC2_pre(t,c,i) + P(t,c)*VARRHO_LC2_pro(t,c,i) ) - 1);
         
                if isreal(POL_LC2(t,c,i))==0
                    POL_LC2(t,c,i) = 0;
                end
            
            %HOMI_LC2(t,c,i) = mean_crime_pre*(1-P(t,c)) + mean_crime_pro*P(t,c) + K_t(YUP(t,c),k,lam)*MU_LC2_draws(t,c,i)*( exp(-alpha(t,c)*POL_LC2(t,c,i)) + P(t,c)*theta_true_LC2(t,c)*(1 - exp(-alpha(t,c)*POL_LC2(t,c,i)))) + eps_LC2_draws(t,c,i); 
             HOMI_LC2(t,c,i) = K_t(YUP(t,c),k,lam)*MU_LC2_draws(t,c,i)*( exp(-alpha(t,c)*POL_LC2(t,c,i)) + P(t,c)*(bias(c)+theta_true_LC2(t,c))*(1 - exp(-alpha(t,c)*POL_LC2(t,c,i)))) + eps_LC2_draws(t,c,i); 
             
                if abs(HOMI_LC2(t,c,i))>=5 && t>=2
                    HOMI_LC2(t,c,i) = HOMI_LC2(t-1,c,i);
                end
      
            DRUNK_LC2(t,c,i) = MU_LC2_draws(t,c,i)*K_t(YUP(t,c),k,lam)./(xchi(2) + exp(alpha(t,c).*POL_LC2(t,c,i)));
            
            OMEGAS(t,c,i) = P(t,c)*(K_t(YUP(t,c),k,lam)*MU_LC2_draws(t,c,i)*(1 - exp(-alpha(t,c)*POL_LC2(t,c,i))));
         
                if counterf==2
                    OMEGAS(t,c,i)=0;
                end
            
            SUM_OMEGAS_SQ(t+1,c,i) = SUM_OMEGAS_SQ(t,c,i) + (OMEGAS(t,c,i)*OMEGAS(t,c,i));
         
            CAP_OMEGAS(t+1,c,i) = 1/((1/var_xi) + ((1/var_q(2))*(SUM_OMEGAS_SQ(t+1,c,i))));
       
            SUM_Q_OMEGAS(t+1,c,i) = SUM_Q_OMEGAS(t,c) + (mean_crime + HOMI_LC2(t,c,i) - K_t(YUP(t,c),k,lam)*MU_LC2_draws(t,c,i)*exp(-alpha(t,c)*POL_LC2(t,c,i)))*OMEGAS(t,c,i);
        
            BEL(t+1,c,i) = CAP_OMEGAS(t+1,c,i)*(((1/var_xi)*BEL(1,c,i)) + ((1/var_q(2))*SUM_Q_OMEGAS(t+1,c,i)));
         
            end
       end
   end
 
  POL_LC2 = sum(POL_LC2,3)/num_draws; 
  DRUNK_LC2 = sum(DRUNK_LC2,3)/num_draws;
  HOMI_LC2 = sum(HOMI_LC2,3)/num_draws; 
   
  BEL_LC2 = sum(BEL,3)/num_draws;                                                            
    BEL_LC2 = BEL_LC2(1:years,:);                                           %Stop in 1933

    for t=2:years
        for c=1:cities
            if abs(BEL_LC2(t,c))>=5
                BEL_LC2(t,c)=BEL_LC1(t-1,c);
            end
        end
    end
 
  CAP_OMEGAS_LC2 = sum(CAP_OMEGAS,3)/num_draws;                                              
    CAP_OMEGAS_LC2 = CAP_OMEGAS_LC2(1:years,:);                             %Stop in 1933
 
    
    
%% %Average Across Latent Classes

z = z';

POL_LC1 = POL_LC1.*(repmat(z(1,:),years,1));
POL_LC2 = POL_LC2.*(repmat(z(2,:),years,1));
p_model = POL_LC1 + POL_LC2;

p_model_pre_mean = sum(sum(p_model.*(1-P)))/sum(sum(1-P));
p_model_pro_mean = sum(sum(p_model.*(P)))/sum(sum(P));


DRUNK_LC1 = DRUNK_LC1.*(repmat(z(1,:),years,1));
DRUNK_LC2 = DRUNK_LC2.*(repmat(z(2,:),years,1));
d_model = DRUNK_LC1 + DRUNK_LC2;

d_model_pre_mean = sum(sum(d_model.*(1-P)))/sum(sum(1-P));
d_model_pro_mean = sum(sum(d_model.*(P)))/sum(sum(P));


HOMI_LC1 = HOMI_LC1.*(repmat(z(1,:),years,1));
HOMI_LC2 = HOMI_LC2.*(repmat(z(2,:),years,1));
q_model = HOMI_LC1 + HOMI_LC2;

q_model_pre_mean = sum(sum(q_model.*(1-P)))/sum(sum(1-P));
q_model_pro_mean = sum(sum(q_model.*(P)))/sum(sum(P));


BEL_LC1 = BEL_LC1.*(repmat(z(1,:),years,1));
BEL_LC2 = BEL_LC2.*(repmat(z(2,:),years,1));
bel_model = BEL_LC1 + BEL_LC2;

bel_model_pre_mean = sum(sum(bel_model.*(1-P)))/sum(sum(1-P));
bel_model_pro_mean = sum(sum(bel_model.*(P)))/sum(sum(P));


CAP_OMEGAS_LC1 = CAP_OMEGAS_LC1.*(repmat(z(1,:),years,1));
CAP_OMEGAS_LC2 = CAP_OMEGAS_LC2.*(repmat(z(2,:),years,1));
cap_omegas_model = CAP_OMEGAS_LC1 + CAP_OMEGAS_LC2;




    
end    
    