% This function computes the Table of estimated Parameters and standard
% errors in Table 2

function [z_bar_mean z_bar_stdev alpha_mean alpha_stdev bias_mean bias_stdev theta_true_mean theta_true_stdev a_mean a_stdev b_mean b_stdev xchi_mean xchi_stdr k k_stdr lam lam_stdr var_q var_q_stdr var_zet var_zet_stdr var_xi var_xi_stdr rho rho_stdr eta eta_stdr Pi_1 Pi_2] = param_table(Abar15_44, E, E_bar, L, R, X_char, R_o, THETA, z, VC)


global years cities latent_classes 

z = kron(z,ones(years,1));

Pi_1 = sum(z(:,1))/(years*cities);

Pi_2 = sum(z(:,2))/(years*cities);


 
        %Recover parameters and densities:
            %Structural parameters as functions of the data
        
            %Common across Latent Classes
                z_bar = log(exp([Abar15_44 E(:,2:3) R L(:,1)]*THETA(1:13)) + 1);
                z_bar_mean  = mean(log(exp([Abar15_44 E(:,2:3) R L(:,1)]*THETA(1:13)) + 1));                                                                                                                                                                    %Mean moral beliefs are a function of religious distribution, time varying as religious distribution changes: R = [baptist orth evangel jew mormon luth metho catho presby] (baptist is ommited category) NxT,1   
                z_bar_stdev = std(log(exp([Abar15_44 E(:,2:3) R L(:,1)]*THETA(1:13)) + 1));
                
                alpha_mean  = mean(1 + log(exp(L*THETA(14:26)) + 1));                                                                                                                                                                                           %Enforcement parameter is a function of: Pre-prohibition trend and L=Dry laws, enfocement law dummy, prohibition field headquarter, prohibition field dummies) NxT,1
                alpha_stdev = std(1 + log(exp(L*THETA(14:26)) + 1));
                
                bias_mean  = mean([R_o L(:,1)]*THETA(27:36));                                                                                                                                                                                                   %Structural prior bias
                bias_stdev = std([R_o L(:,1)]*THETA(27:36));
   
                
            %Type Specific    
                theta_true_mean = mean(log(exp([X_char(:,31:33) Abar15_44 E_bar(:,2:3) L(:,1)]*reshape(THETA(36 + 8*latent_classes + 1:36 + 15*latent_classes)*1e-1,7,latent_classes)) + 1));                                                                   %Structural elasticity of crime to Prohibition
                theta_true_stdev = std(log(exp([X_char(:,31:33) Abar15_44 E_bar(:,2:3) L(:,1)]*reshape(THETA(36 + 8*latent_classes + 1:36 + 15*latent_classes)*1e-1,7,latent_classes)) + 1));
            
                a_mean = mean(log(exp(repmat(THETA(36 + 6*latent_classes + 1:36 + 7*latent_classes)',years*cities,1)) + 1) + 1);                                                                                                                                %Wetness distribution parameter (correlated with moral views)
                a_stdev = std(log(exp(repmat(THETA(36 + 6*latent_classes + 1:36 + 7*latent_classes)',years*cities,1)) + 1) + 1);
                
                b_mean = mean(log(exp(repmat(THETA(36 + 7*latent_classes + 1:36 + 8*latent_classes)',years*cities,1).*repmat(z_bar,1,latent_classes)) + 1) + 1);                                                                                                %Wetness distribution parameter (correlated with moral views)                         
                b_stdev = std(log(exp(repmat(THETA(36 + 7*latent_classes + 1:36 + 8*latent_classes)',years*cities,1).*repmat(z_bar,1,latent_classes)) + 1) + 1);
                
            
             %Other Parameters (Standard Errors Using the Delta Method)                
                
                xchi1 = exp(THETA(36 + 1));
                xchi2 = exp(THETA(36 + latent_classes));
                xchi_mean = (Pi_1*(log(xchi1 + 1))) + (Pi_2*(log(xchi2 + 1)));                                                                                                                                                                                  %Drunkard Arrest enforcement parameter                             
                xchi_stdr = sqrt([Pi_1*xchi1/(1+xchi1) Pi_2*xchi2/(1+xchi2)]*VC(36 +1:36 + latent_classes,36 +1:36 + latent_classes)*[Pi_1*xchi1/(1+xchi1) Pi_2*xchi2/(1+xchi2)]');
                
                eta1 = exp(THETA(36 + 15*latent_classes + 1));
                eta2 = exp(THETA(36 + 16*latent_classes));
                eta = (Pi_1*eta1./(1 + eta1)) + (Pi_2*eta2./(1 + eta2));                                                                                                                                                                                        %Marginal utility of public goods
                eta_stdr = sqrt( [Pi_1*(eta1/((1 + eta1)^2))  Pi_2*(eta2/((1 + eta2)^2))]*VC(36 + 15*latent_classes + 1:36 + 16*latent_classes,36 + 15*latent_classes + 1:36 + 16*latent_classes)*[Pi_1*(eta1/((1 + eta1)^2))  Pi_2*(eta2/((1 + eta2)^2))]');
        
                k1 = exp(THETA(36 + latent_classes + 1));               
                k = k1./(1 + k1);                                                                                                                                                                                                                               %Speed of alcohol-market catchup parameter-inverse of trough                
                k_stdr = sqrt(VC(36 + latent_classes + 1,36 + latent_classes + 1)*((k1/((1 + k1)^2))^2));
                
                
                lam1 = exp(THETA(36 + 2*latent_classes + 1));
                lam = lam1./(1 + lam1);                                                                                                                                                                                                                         %Impact fall in alcohol market parameter 
                lam_stdr = sqrt(VC(36 + 2*latent_classes + 1,36 + 2*latent_classes + 1)*((lam1/((1 + lam1)^2))^2));
                
                vq1 = exp(THETA(36 + 3*latent_classes + 1));
                vq2 =exp(THETA(36 + 4*latent_classes));
                var_q = (Pi_1*(log(vq1 + 1))) + (Pi_2*(log(vq2 + 1)));                                                                                                                                                                                          %Variance of the error in the crime equation
                var_q_stdr = sqrt([Pi_1*vq1/(1+vq1) Pi_2*vq2/(1+vq2)]*VC(36 + 3*latent_classes + 1:36 + 4*latent_classes,36 + 3*latent_classes + 1:36 + 4*latent_classes)*[Pi_1*vq1/(1+vq1) Pi_2*vq2/(1+vq2)]');
                                          
                var_xi = 0.1;                                                                                                                                                                                                                                   %Normalization: Variance of Prior Biases
                var_xi_stdr = 0;
                
                vz1 = exp(THETA(36 + 4*latent_classes + 1));
                vz2 = exp(THETA(36 + 5*latent_classes));
                var_zet = ((Pi_1*(log(vz1 + 1))) + (Pi_2*(log(vz2 + 1))))/var_xi;                                                                                                                                                                               %Variance of the distribution of moral shocks
                var_zet_stdr = sqrt([Pi_1*vz1/(1+vz1) Pi_2*vz2/(1+vz2)]*VC(36 + 4*latent_classes + 1:36 + 5*latent_classes,36 + 4*latent_classes + 1:36 + 5*latent_classes)*[Pi_1*vz1/(1+vz1) Pi_2*vz2/(1+vz2)]');
                
                
                rh1 = exp(THETA(36 + 5*latent_classes + 1));
                rh2 = exp(THETA(36 + 6*latent_classes));
                rho = (Pi_1*(rh1-1)./(1 + rh1)) + (Pi_2*(rh2-1)./(1 + rh2));                                                                                                                                                                                    %Correlation between morals and prior beliefs        
                rho_stdr = sqrt( [Pi_1*2*rh1/((1+rh1)^2)  Pi_2*2*rh2/((1+rh2)^2)]*VC(36 + 5*latent_classes + 1:36 + 6*latent_classes,36 + 5*latent_classes + 1:36 + 6*latent_classes)*[Pi_1*2*rh1/((1+rh1)^2)  Pi_2*2*rh2/((1+rh2)^2)]');
                
     


end