function [bel, cap_omega] = public_beliefs(p,d,q,mean_crime,P,alpha,bias,var_xi,var_q,xchi)                             %This function computes the posterior public beliefs and cap_omega
                                                                                                                        %Notice this function returns a vector, the public beliefs in city c for all years
                                                                                                                        %Inputs are:
                                                                                                                        %Police vector of city: p (T,1)
                                                                                                                        %Drunk arrests vector of city: d (T,1)
                                                                                                                        %Homicide rate of city: q (T,1)
                                                                                                                        %Years Under Prohibition vector of city: YUP (T,1)
                                                                                                                        %Prohibition Year Indicator: P (T,1)
                                                                                                                        %theta_true of city c (scalar)
                                                                                                                        %Variance of belief biases var_xi
                                                                                                                        %Variance of crime shock var_q
                                                                                                                        %Enforcement parameter: alpha
                                                                                                                        %Arrests parameter: xchi
                                                                                                                        %Market catchup parameter: k
                                                                                                                        %years: number of years in panel (1,1)
                                                                                                                        %k parameter                                                                                                 
                                                                                                                        %year: t

                                                                                                                        
global BIG                                                                                                              %The matrix BIG has in each row t ones for years tau<t, to add all past realizations          
    

omega = P.*(d.*(xchi + exp(alpha.*p)).*(1 - exp(-alpha.*p)));                                                           %Vector of omegas (CxT,1) (Multiply by P to exclude from the summation years under No Prohibition)

omegas_sq = (omega.*omega);                                                                                             %Compute vector of squared omegas

sum_omegas_sq = BIG*omegas_sq;                                                                                          %Creates the vector of partial SUMS of squared omegas up to year t-1 for each year and city

cap_omega = 1./((1./var_xi) + ((1./var_q).*sum_omegas_sq));                                                             %This is the vector of cap_omegas of all cities

q_omegas = ((q+mean_crime) - (d.*(xchi + exp(alpha.*p)).*exp(-alpha.*p))).*omega;                                       %Multiply q (net of non-prohibition crime) and omega element by element

sum_q_omegas = BIG*q_omegas;                                                                                            %Creates vector of sum of q times omegas from initial Prohibition year to immediately previous year, for each year

%Computes the common mean belief (times variance)
                                                                                                                        %Posterior mean belief is a linear combination of past sequence of signals and prior
 bel = cap_omega.*(((1./var_xi).*(bias)) + ((1./var_q).*sum_q_omegas));                                                 %Common belief already multiplied by Cap_omega
 
 
 
 
 
 