%This function computes the likelihood with the infenitesimal variation to
%calculate the score needed for the Analytic Variance Covariance of the MLE
%estimator


function [sc] = score(z, p, d, q, mean_crime, YUP, P, R, X_char, Abar15_44, E, E_bar, R_o, L, PAR)

global years cities latent_classes

z = kron(z,ones(years,1));

    %Structural parameters as functions of the data
        
        %Common across Latent Classes
        z_bar = log(exp([Abar15_44 E(:,2:3) R L(:,1)]*PAR(1:13)) + 1);                                                                                                    %Mean moral beliefs are a function of religious distribution, time varying as religious distribution changes: R = [baptist orth evangel jew mormon luth metho catho presby] (baptist is ommited category) NxT,1   
            
        alpha = 1 +  log(exp(L*PAR(14:26)) + 1);                                                                                                                          %Enforcement parameter is a function of: Pre-prohibition trend and L=Dry laws, enfocement law dummy, prohibition field headquarter, prohibition field dummies) NxT,1
        
        bias = [R_o L(:,1)]*PAR(27:36);                                                                                                                                   %Structural prior bias
        
       
        %Latent-Class specific                
        xchi = log(exp( ( repmat(PAR(36 + 1:36 + latent_classes)',years*cities,1) ) ) + 1);                                                                               %Drunkard Arrest enforcement parameter                             
        
        k = exp(PAR(36 + latent_classes + 1:36 + 2*latent_classes)')./(1 + exp(PAR(36 + latent_classes + 1:36 + 2*latent_classes)') );                                    %Speed of alcohol-market catchup parameter-inverse of trough 
            
        lam = exp(PAR(36 + 2*latent_classes + 1:36 + 3*latent_classes)')./(1 + exp(PAR(36 + 2*latent_classes + 1:36 + 3*latent_classes)') );                              %Impact fall in alcohol market parameter 

        var_q = log(exp(PAR(36 + 3*latent_classes + 1:36 + 4*latent_classes)') + 1);                                                                                      %Variance of the error in the crime equation
        
        var_zet = log(exp(PAR(36 + 4*latent_classes + 1:36 + 5*latent_classes)') + 1);                                                                                    %Variance of the distribution of moral shocks
       
        rho = ((exp(PAR(36 + 5*latent_classes + 1:36 + 6*latent_classes)')-1)./(1+exp(PAR(36 + 5*latent_classes + 1:36 + 6*latent_classes)')));                           %Correlation between morals and prior beliefs
        
        a = log(exp(repmat(PAR(36 + 6*latent_classes + 1:36 + 7*latent_classes)',years*cities,1)) + 1) + 1;                                                               %Wetness distribution parameter (correlated with moral views)                                                                                                                                                                            
              
        b = log(exp(repmat(PAR(36 + 7*latent_classes + 1:36 + 8*latent_classes)',years*cities,1).*repmat(z_bar,1,latent_classes)) + 1) + 1;                               %Wetness distribution parameter (correlated with moral views)
                    
        theta_true = (log(exp([X_char(:,31:33) Abar15_44 E_bar(:,2:3) L(:,1)]*reshape(PAR(36 + 8*latent_classes + 1:36 + 15*latent_classes),7,latent_classes)) + 1));     %Structural elasticity of crime to Prohibition                                                                                                 
        
        eta = exp(PAR(36 + 15*latent_classes + 1:36 + 16*latent_classes)')./(1 + exp(PAR(36 + 15*latent_classes + 1:36 + 16*latent_classes)'));                           %Marginal utility of public goods

        %Normalization                                                                                                                                         
        var_xi = 0.1;                                                                                                                                                     %Variance of belief biases 
        
        
        
        
%%      %Computation of the Expected Likelihood function summing across cities

EL=zeros(years*cities,latent_classes);

    for i = 1:latent_classes

        %Compute the vector of common beliefs sequences and belief weights sequences
        [beliefs, cap_omega] = public_beliefs(p,d,q,mean_crime,P,alpha,bias,var_xi,var_q(i),xchi(i));
        
        varrho_med = g_varrho(p,YUP,P,beliefs,alpha,a(:,i),b(:,i),k(i),lam(i),z_bar,eta(i));
        
        g_eps = g_epsilon(p,d,q,P,theta_true(:,i),alpha,xchi(:,i)); 


    f = (1/(sqrt(2*pi*var_q(i)))).*(exp(-(1/(2*var_q(i))).*((g_eps).^2))).*((density_varrho_N(varrho_med,a(:,i),b(:,i),var_zet(i))).^(1-P)).*((density_varrho_P(varrho_med,a(:,i),b(:,i),var_zet(i),var_xi,cap_omega,rho(i))).^(P)).*jacob(p,YUP,alpha,k(i),lam(i),eta(i));

    EL(:,i) = z(:,i).*log(f);

    end

sc = sum(EL,2);

end


